import { createApp } from 'vue';
const toObject = (arr) => arr.reduce((prev, cur) => {
    prev[cur.name] = cur.fn;
    return prev;
}, {});
/* tslint:disable */
export class VueRenderer {
    onInitialize(actor) {
        const localActor = actor;
        let data;
        let template;
        try {
            data = localActor.data();
            template = localActor.template;
        }
        catch (_) {
            return;
        }
        if (typeof template !== 'function' || typeof data !== 'object') {
            return;
        }
        const internalFunctions = ['data', 'template', 'constructor'];
        const isInternalFunction = (key) => internalFunctions.includes(key);
        const allEvents = (props) => Object.entries(props)
            .filter(([key]) => key.startsWith('on'))
            .map(([key, value]) => ({ [key.substr(2).toLowerCase()]: value }))
            .reduce((prev, newv) => (Object.assign(Object.assign({}, prev), newv)), {});
        const withoutEvents = (props) => Object.entries(props)
            .filter(([key]) => !key.startsWith('on'))
            .reduce((prev, newv) => (Object.assign(Object.assign({}, prev), newv)), {});
        function createElementProxy(createElement) {
            return function (el, props, ...children) {
                const mappedProps = { attrs: withoutEvents(props || {}), on: allEvents(props || {}) };
                return createElement(el, mappedProps, children);
            };
        }
        const methods = Object.getOwnPropertyNames(localActor.constructor.prototype).filter((key) => typeof localActor.constructor.prototype[key] === 'function' && !isInternalFunction(key));
        localActor.__internals = localActor.__internals || {};
        localActor.__internals.vue = createApp({
            data() {
                return { state: data };
            },
            el: `#${localActor.id}`,
            methods: toObject(methods
                .filter((method) => localActor.self[method])
                .map((method) => {
                return { name: method, fn: localActor.self[method] };
            })),
            render(createElement) {
                return template.call(this, createElementProxy(createElement));
            },
        });
    }
    onBeforeMessage(actor, message) {
        //
    }
    onAfterMessage(actor, message) {
        //
    }
    onError(actor, message, error) {
        //
    }
}
